Player = {}

function Player:load()

    self.x = love.graphics.getWidth() / 2
    self.y = love.graphics.getHeight() / 2 - 62
    self.spriteSheet = love.graphics.newImage("Sprites/TheWizzl.png")
    self.speed = 400

    self.ground = self.y
    self.yVel = 0
    self.jump_height = -300
    self.gravity = -500

    -- Animation Information

    local grid = anim8.newGrid( -- No need to store this function in the table
        32,32, self.spriteSheet:getWidth(), self.spriteSheet:getHeight()
    )

    self.anim = {}
    self.anim.idle = anim8.newAnimation(grid('1-4', 1), 0.2)
    self.anim.run = anim8.newAnimation(grid('8-11', 1), 0.1)
    self.anim.jump = anim8.newAnimation(grid(19, 1), 0.2)
    self.anim.fall = anim8.newAnimation(grid(20, 1), 0.2)
    self.anim.hide = anim8.newAnimation(grid('22-27', 1), 0.1)
    self.anim.hat = anim8.newAnimation(grid(28, 1), 0.1)
    self.anim.hatWalk = anim8.newAnimation(grid('29-33', 1), 0.1)
    self.anim.getUp = anim8.newAnimation(grid('35-38', 1), 0.1)

    self:setState("idle")
    self.on_anim = false    -- I added this so that the movement does not break the animation of getting up or lying down
    self.crouch = false
    self.switch = 2

end

function Player:setState(state)

    if state ~= self.state then

        self.anim_curr = self.anim[state]
        self.anim_curr:gotoFrame(1)

        self.prev_state = self.state
        self.state = state

    end

end

function Player:update(dt)

    if love.keyboard.isDown("s") then

        if not self.crouch then
            self:setState("hide")
            self.anim_on = true
            self.crouch = true
        end

    end

    if love.keyboard.isDown("space") then

        if self.crouch then
            self:setState("getUp")
            self.anim_on = true
            self.crouch = false
        end

    end

    if love.keyboard.isDown("escape") then
        love.event.quit()
    end

    -- State management --

    local state = self.state

    if state == "hide" then -- You can manage behaviors related to states here

        if self.anim_curr.position == 6 then
            self:setState("hat")
            self.anim_on = false
        end

    elseif state == "getUp" then

        if self.anim_curr.position == 3 then
            self:setState("idle")
            self.anim_on = false
        end

    end

    -- Movement and anim update --

    Player:move(dt)
    Player:jump(dt)

    self.anim_curr:update(dt)

end

function Player:draw()
    self.anim_curr:draw(self.spriteSheet, self.x, self.y, nil, self.switch, 2, 16)
end

function Player:move(dt)

    local onMove;

    if love.keyboard.isDown("a") then
        self.x = self.x - self.speed * dt
        self.switch = -2
        onMove = true

    elseif love.keyboard.isDown("d") then
        self.x = self.x + self.speed * dt
        self.switch = 2
        onMove = true

    end

    if not self.anim_on then

        if onMove then
            self:setState((self.crouch) and "hatWalk" or "run")
        else
            self:setState((self.crouch) and "hat" or "idle")
        end

    end

end

function Player:jump(dt) -- I haven't touched this function.

    if self.crouch == false then

        if love.keyboard.isDown("w") then
            if self.yVel == 0 then
                self.yVel = self.jump_height
            end
        end

        if self.yVel ~= 0 then
            self.y = self.y + self.yVel * dt
            self.yVel = self.yVel - self.gravity * dt
        end

        if self.y > self.ground then
            self.yVel = 0
            self.y = self.ground
        end

        if self.yVel < 0 then
            self.anim_curr = self.anim.jump
        elseif self.yVel > 0 then
            self.anim_curr = self.anim.fall
        end

    end

end

